From keeler1@payne.mps.ohio-state.edu Fri Jun 11 14:12:16 1993
Date: Fri, 11 Jun 93 14:11:08 EDT
From: Keeler Guest Account <keeler1@payne.mps.ohio-state.edu>
To: pogge@payne.mps.ohio-state.edu
Subject: latest working version of chi-auto

        program chi_auto

*  Automatic scaling of spectra assuming constant flux in certain lines.
*
*  Version 1.0, August 1991
*  Version 1.1, May 1992, now executable on SUN, Apollo and VAX/VMS
*  Version 2.0, Jun 1993, included possibility of masking a wavelength
*                         region.     /IW @ STScI
*  Version 2.1, Jun 1993, write output in formatted way. N.B. this
*                         format might have to be canged for specific
*                         applications     /IW @ STScI
*  Version 2.2, Jun 1993, more than one masked region now possible
*                         /IW @ OSU
*  Version 2.3, Jun 1993, added small rescaling code written by
*                         Rick Pogge (OSU) to scale spectra always to
*                         approx. unity before doing the main scaling
*                         and rescale them afterwards to the original
*                         scale. This is to avoid small numbers.
*                         /IW @ OSU
*
*
*  Ref: van Groningen & Wanders, 1992: P.A.S.P. 104, p 700 (Aug 92)
*
*  Ernst van Groningen, Ignaz Wanders
*  Uppsala Astronomical Observatory
*  Box 515
*  S-75120 Uppsala
*  Sweden                  
*     e-mail: 21619::LABAN::ERNST (SPAN)    ernst@astro.uu.se  (INTERNET)
*             21619::LABAN::IGNAZ (SPAN)    ignaz@astro.uu.se  (INTERNET)
*
*  The programme fits a simple function to the difference between the 
*  2 spectra in the specified wavelength range, and calculates the 
*  chisq of this fit. By varying the flux factor, wavelength shift and
*  FWHM of the resolution of the second spectrum the minimum chisq is
*  searched for, according to a grid search method (see Bevington, Data
*  reduction and error analysis for the physical sciences, McGraw-Hill,
*  1969, page 208ff). The search is performed twice, either convolving
*  the first or the second spectrum. The lowest value found is used to
*  calculate the output file. The programme stops when the relative change 
*  in chisq from one step to the next is less than the "stopping criterium"
*
*
*  The programme prompts for the following input parameters
*        > reference spectrum with 3 columns, lambda,flux,rms
*        > spectrum to be scaled, same format
*        > start wavelength of fitting region
*        > end wavelength of fitting region  
*        > start wavelength of masked region
*        > end wavelength of masked region
*        > initial estimate of flux scale factor
*        > initial estimate of FWHM of spectral convolution (pixels)
*        > initial estimate of wavelength shift (pixels)
*        > step size in flux factor
*        > step size in FWHM
*        > step size in shift
*        > stopping criterium
*        > name of output file name
*
*
*  A summary of the results is written to the file 'results' (in 
*  routine WRITE_RES). The 4 columns contain:
*        > the flux ratio (i.e. for scaling the second input file)
*        > the FWHM ( if FWHM>0 the second file is convolved, if
*            FWHM<0 the first file)
*        > the wavelength shift (applied on the second file)
*        > the value of chisq in the minimum
*
*  The modified spectra are written to the specified output file which
*  contains seven columns, whose contents depend on the sign of the
*  FWHM (see above)                                           
*
*  For FWHM > 0
*        1. wavelength
*        2. second spectrum: scaled, shifted and convolved
*        3. one sigma rms of column 2
*        4. identical to column 2
*        5. second spectrum: scaled, shifted (not convolved)
*        6. difference between reference spectrum and column 2
*        7. difference between reference spectrum and column 5
*
*  For FWHM < 0
*        1. wavelength
*        2. second spectrum: scaled, shifted (not convolved)
*        3. one sigma rms of column 2
*        4. convolved reference spectrum
*        5. non modified reference spectrum
*        6. difference between column 4 and column 2
*        7. difference between column 5 and column 2
*
*  
*  CPU time usage depends very much on how good the initial guesses are and 
*  on the step sizes. Some tests with 'typical' input parameters yielded:
*
*       Apollo workstation  -	DN3550    140 secs
*				DN10000     4 secs
*	VAX 4300		           16 secs
*       SUN Sparc-2                       ~12 secs
*  
* Bugs:
*     It appears as if the program crashes on a singular matrix if both
*     backgrounds in the two spectra are (close to) zero. Because the
*     program does not care about any constant backgrounds, this problem
*     can be solved by adding an artificial constant to one of the spectra
*     before running the program and subtracting it after program
*     completion.             /IW 4 June 1993
*

        implicit none          

        integer nmax, ntot, status, nmask_max

	parameter (nmask_max=5)
*	       maximum expected masked regions

        integer n1,n2    
*              pixel numbers of starting and end of fitting region 
	integer m1(nmask_max),m2(nmask_max),nmask
*	       pixel numbers of starting and end of masked regions

        parameter        (nmax=1024)            
*                        maximum spectral length 
*			 (see also subroutine fit_poly!)

        real wave(nmax)
        real flux_1(nmax),sig_1(nmax)  
*            original (scaled) fluxes and sigmas of first file 
        real flux_2(nmax),sig_2(nmax)  
*            original (scaled) fluxes and sigmas of second file 
        real r0,f0,d0     
*            starting values for flux ratio, FWHM and lambda shift 
        real dr,df,dd     
*            initial step size for these three parameters 
        real r,f,d        
*            results for these three parameters 
        real stop         
*            stop if delta(chisq) < stop * ntot  
        real chisq, fsc


C-----------------------------------------------------------------------
C    Read spectra from disk

        call read_data(wave,flux_1,sig_1,flux_2,sig_2,ntot,fsc)
         
C----------------------------------------------------------------------- 
C    Inquire for lambda region and find start and end pixel for extraction

        call read_lamb(wave,ntot,n1,n2,m1,m2,nmask,nmask_max)                                        

C----------------------------------------------------------------------- 
C    Inquire for starting values, initial stepsizes and stopping criterium

        call read_init(r0,f0,d0,dr,df,dd,stop)           

C----------------------------------------------------------------------- 
C    Locate the position of the minimum chisq

        call find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >          m1,m2,nmask,r0,f0,d0,dr,df,dd,stop,r,f,d,chisq,status)


C----------------------------------------------------------------------- 
C    Output the position of the minimum and the value of chisq

       call write_res(r,f,d,chisq)

C----------------------------------------------------------------------- 
C    Output the resulting spectra to disk

       call wrout(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d,fsc)

       stop
       end

************************* END OF MAIN PROGRAM *************************


        SUBROUTINE READ_DATA (wave,flux_1,sigma_1,flux_2,sigma_2,n,fsc)

        implicit         none

        real             wave(1),flux_1(1),flux_2(1),mean1,mean2
        real             sigma_1(1),sigma_2(1),scale1,scale2
	real		 fsc, farg
        integer          n, nmax, i, n1, n2, npts, ifsc
                      
        character*40     infile_1,infile_2

        logical          exist 

        parameter        (nmax=1024)
        
        real             wave_1(nmax), wave_2(nmax)
C-----------------------------------------------------------------------
C   Prompt for filename of first input file

	exist = .false.

	do while (.not.exist)
       	    write (6,'(" Enter name of first spectrum : ",$)')
       	    read (5,'(a)') infile_1

            inquire (file=infile_1,exist=exist)
            if (.not.exist) then
                write (6,*) ' '
                write (6,*) ' *** WARNING ***  File not found '  
                write (6,*) ' '                              
	    endif
	enddo

C-----------------------------------------------------------------------
C   Prompt for filename of second input file

	exist = .false.

	do while (.not.exist)
            write (6,'(" Enter name of second spectrum : ",$)')
            read (5,'(a)') infile_2

            inquire (file=infile_2,exist=exist)
            if (.not.exist) then
                write (6,*) ' '
                write (6,*) ' *** WARNING ***  File not found '  
                write (6,*) ' '                              
	    endif
	enddo
              
        open(unit=40, file=infile_1, status='old')
        do i = 1,nmax
            read(40,*,end=1) wave_1(i),flux_1(i),sigma_1(i)
        enddo                                    
  001   n1 = i-1
        close(unit=40)

        open(unit=40, file=infile_2, status='old')
        do i = 1,nmax
            read(40,*,end=2) wave_2(i),flux_2(i),sigma_2(i)
        enddo               
  002   n2 = i-1
        close(unit=40)

        if (n1.ne.n2) stop
     &      ' *** FATAL ***  Files have different lengths! '

        n = n1
        
        do i = 1,n
            wave(i) = wave_1(i)
            if (wave_1(i).ne.wave_2(i)) stop
     >        ' *** FATAL ***  Files have different wavelength scales! '
        enddo

*** code written by R. Pogge (OSU) begin ***

C  Autoscale the fluxes to of order unity based on the mean exponent of
C  the reference spectrum.   Remember the flux scaling factor for later
C  to rescale the output fluxes (wrout2.f) to the original flux scale.

      fsc = 0.0
      npts = 0
      do 400 i = 1, n
         farg = abs(flux_1(i))
         if (farg .ne. 0.0) then
            fsc = fsc + float(nint(log10(farg)))
            npts = npts + 1
         end if
 400  continue
      fsc = fsc / float(npts)
      if (abs(fsc) .lt. 3.) then
         fsc = 1.0
         return
      else
         ifsc = nint(fsc) - 1
         fsc = 10.0**(-ifsc)
      end if
      do 401 i = 1, n
         flux_1(i) = fsc*flux_1(i)
         sigma_1(i) = fsc*sigma_1(i)
         flux_2(i) = fsc*flux_2(i)
         sigma_2(i) = fsc*sigma_2(i)
 401  continue

*** code written by R. Pogge (OSU) end ***

* supply the user with an initial guess of the scaling factor

	mean1 = 0.
	mean2 = 0.
	do i = 1, n
	   mean1 = mean1 + flux_1(i)
	   mean2 = mean2 + flux_2(i)
	enddo
	scale1 = mean1/float(n)
	scale2 = mean2/float(n)
	write(6,*)' Initial guess of scaling parameter would be ',
     &                 scale1/scale2

        return
        end 
         
C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 

        SUBROUTINE WROUT(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d,fsc)

        character*40     outfile, karakter*1

        logical          exist 
                              
        real wave(1)
        real flux_1(1),sig_1(1)
        real flux_2(1),sig_2(1)
        real r,f,d

        integer ntot,i,nmax

        parameter        (nmax=1024)            
*                        maximum spectral length 

        real work1(nmax),work2(nmax),work3(nmax)
        real work4(nmax),work5(nmax),work6(nmax)

C-----------------------------------------------------------------------
C   Prompt for filename of output file

	exist = .true.

	do while (exist)
       	    write (6,'(" Enter name of output spectrum file : ",$)')
       	    read (5,'(a)') outfile

            inquire (file=outfile,exist=exist)
            if (exist) then
                write (6,*) ' '
                write (6,*) ' *** WARNING ***  Already exists '  
                write (6,*) ' '                 
123		write(6,'('' Override?  (y/n)  > '',$)')
		read(5,'(a1)',err=123) karakter    
		if (karakter.eq.'y'.or.karakter.eq.'Y') exist = .false.         
	    endif
	enddo

C-----------------------------------------------------------------------
C   Shift second spectrum by 'd' pixels

        call shift_spec(flux_2,work1,ntot,d,.false.)
        call shift_spec(sig_2,work2,ntot,d,.true.)

C-----------------------------------------------------------------------
C   Scale second spectrum with flux factor 'r'                          
        do i = 1,ntot
           work1(i) = r*work1(i)
           work2(i) = r*work2(i)
        enddo
   
C-----------------------------------------------------------------------

C   Convolve (first/second) file with gaussian FWHM 'f' depending
C   whether f is (negative/positve)

        if (f.ge.0.0) then 
           call conv_gauss(work1,work3,f/2.35482,ntot,.false.)
           call conv_gauss(work2,work4,f/2.35482,ntot,.true.)
        else
           call conv_gauss(flux_1,work5,f/2.35482,ntot,.false.)
           call conv_gauss(sig_1,work6,f/2.35482,ntot,.true.)
           do i = 1,ntot
              work3(i) = work1(i)
              work4(i) = work2(i)
           enddo
        endif

* rescale the spectra to the original scale before writing the 
* output.     (as suggested by R. Pogge (OSU)  )

	do i = 1, ntot
	   flux_1(i) = flux_1(i)/fsc
	   sig_1(i)  = sig_1(i)/fsc
	   flux_2(i) = flux_2(i)/fsc
	   sig_2(i)  = sig_2(i)/fsc
	   work1(i)  = work1(i)/fsc
	   work2(i)  = work2(i)/fsc
	   work3(i)  = work3(i)/fsc
	   work4(i)  = work4(i)/fsc
	   work5(i)  = work5(i)/fsc
	   work6(i)  = work6(i)/fsc
	enddo

C-----------------------------------------------------------------------
C   Open the output file 

        open (unit=38, file=outfile, status='unknown')
        
        if (f.ge.0.0) then 
           do i = 1,ntot
              write(38,90) wave(i),work3(i),work4(i),work3(i),work1(i),
     >                    flux_1(i)-work3(i),flux_1(i)-work1(i)
           enddo
        else
           do i = 1,ntot
              write(38,90) wave(i),work3(i),work4(i),work5(i),flux_1(i),
     >                    work5(i)-work3(i),flux_1(i)-work3(i)
           enddo
        endif

        return
90	format(f10.2,1x,e11.5,1x,e11.5,1x,e11.5,1x,e11.5,
     &                                       1x,e11.5,1x,e11.5)

        end

C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
        SUBROUTINE READ_LAMB(wave,ntot,n1,n2,m1,m2,nmask,nmask_max)          

        implicit none
	integer nmask_max
        integer ntot,n1,n2,i,j,m1(nmask_max),m2(nmask_max),nmask
        real wave(ntot)
        real wave_min,wave_max, mask_min, mask_max, temp

        write(6,'(" The wavelength range is from ",f9.2," to ",f9.2)')
     >               wave(1),wave(ntot)

  001   write(6,'(" Enter required wavelength range: lambda min: ",$)') 
        read(5,*,err=1) wave_min

  002   write(6,'(" Enter required wavelength range: lambda max: ",$)') 
        read(5,*,err=2) wave_max

C    Find (approximate) pixel numbers of these wavelengths
                              
        n1 = 1
        n2 = ntot
        do i = 1,ntot
            if (wave(i).lt.wave_min) n1 = i
            if (wave(i).le.wave_max) n2 = i 
        enddo

5	write(6,'('' Enter number of masked regions > '',$)')
	read(5,*,err=5) nmask
	if (nmask.gt.nmask_max) then
	   print*,'max. nr. of masked regions set to ',nmask_max
	   nmask = nmask_max
	endif

* if no masked regions define a dummy one

	if (nmask.le.0) then
	   m1(1) = 0
	   m2(1) = 0
	   nmask = 1
	endif

	do j = 1, nmask
10	   write(6,'(
     &     '' Enter start of masked region #'',i2,'' > '',$)') j
	   read(5,*,err=10) mask_min
20	   write(6,'(
     &     '' Enter   end of masked region #'',i2,'' > '',$)') j
	   read(5,*,err=20) mask_max
	   if (mask_max.lt.mask_min) then
	      temp = mask_min
	      mask_min = mask_max
	      mask_max = temp
	   endif
	   m1(j) = 1
	   m2(j) = ntot
	   do i = 1, ntot
	      if (wave(i).lt.mask_min) m1(j) = i
	      if (wave(i).le.mask_max) m2(j) = i
	   enddo
	enddo

        return
        end

C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 
        SUBROUTINE READ_INIT(r0,f0,d0,dr,df,dd,stop)    

        implicit none

        real r0,f0,d0,dr,df,dd,stop

  001   write(6,'(" Enter initial flux ratio: ",$)')
        read(5,*,err=1) r0

  002   write(6,'(" Enter initial convolution factor: ",$)')
        read(5,*,err=2) f0

  003   write(6,
     &      '(" Enter initial wavelength shift (pixel units): ",$)')
        read(5,*,err=3) d0

  004   write(6,'(" Enter stepsize in flux ratio: ",$)')
        read(5,*,err=4) dr

  005   write(6,'(" Enter stepsize in convolution factor: ",$)')
        read(5,*,err=5) df

  006   write(6,
     &   '(" Enter stepsize in wavelength shift (pixel units): ",$)')
        read(5,*,err=6) dd
  
  007   write(6,'(" Enter stopping criterium: ",$)')
        read(5,*,err=7) stop

        return
        end



C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 
        SUBROUTINE FIND_MIN(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >          m1,m2,nmask,r0,f0,d0,dr,df,dd,stop,r,f,d,chisq0,stat)
     

        implicit none

	integer nmask
        integer n1,n2,ntot,max_iter,count,stat,m1(nmask),m2(nmask)
        real wave(ntot)
        real flux_1(ntot),sig_1(ntot)
        real flux_2(ntot),sig_2(ntot)
        real r0,f0,d0,dr,df,dd,r,f,d,stop,chisq 
        real rnew,fnew,dnew,chisq0
        real rtemp,ftemp,dtemp,chitemp
        logical proceed

        parameter (max_iter=30)

        stat = 0               
*       minimum found if stat not changed 
        count = 0

        proceed = .true.
        r = r0
        f = f0
        d = d0
        
        chisq0 = 1e6

C----------------------------------------------------------------------- 
C   Start the grid search 

        do while (proceed)
        count = count + 1
            
C----------------------------------------------------------------------- 
C   Find minimum in chisq by varying r and constant f and d. 

            call find_min_r(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >                                m1,m2,nmask,r,f,d,dr,rnew,chisq)
            r = rnew

C----------------------------------------------------------------------- 
C   Find minimum in chisq by varying d and constant r and f. 

           call find_min_d(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >                               m1,m2,nmask,r,f,d,dd,dnew,chisq)
            d = dnew
                
C----------------------------------------------------------------------- 
C   Find minimum in chisq by varying f and constant r and d. 

            call find_min_f(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >                               m1,m2,nmask,r,f,d,df,fnew,chisq)
            f = fnew

C----------------------------------------------------------------------- 

            if(abs(chisq-chisq0).lt.stop*ntot) proceed = .false.
            if(count.ge.max_iter) then
               proceed = .false.
               stat=1
            endif
            chisq0 = chisq
        enddo

C----------------------------------------------------------------------- 
C   Store the values of r, f, d and chisq and interchange the files
C   to check if a lower minimum is found when the other file is
C   convolved

        rtemp = r
        ftemp = f
        dtemp = d
        chitemp = chisq0

C----------------------------------------------------------------------- 
C   Repeat procedure with the files interchanged

        count = 0

        proceed = .true.
        r = r0
        f = f0
        d = d0

        chisq0 = 1e6

C----------------------------------------------------------------------- 
C   Start the grid search 

        do while (proceed)
        count = count + 1
            
C----------------------------------------------------------------------- 
C   Find minimum in chisq by varying r and constant f and d. 

            call find_min_r(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     >                                m1,m2,nmask,r,f,d,dr,rnew,chisq)
            r = rnew

C----------------------------------------------------------------------- 
C   Find minimum in chisq by varying d and constant r and f. 

           call find_min_d(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     >                              m1,m2,nmask,r,f,d,dd,dnew,chisq)
            d = dnew
                
C----------------------------------------------------------------------- 
C   Find minimum in chisq by varying f and constant r and d. 

            call find_min_f(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     >                              m1,m2,nmask,r,f,d,df,fnew,chisq)
            f = fnew

C----------------------------------------------------------------------- 

            if(abs(chisq-chisq0).lt.stop*ntot) then
               proceed = .false.
               stat = 0
            endif
            if(count.ge.max_iter) then
               proceed = .false.
            endif
            chisq0 = chisq
        enddo

        if (chitemp.lt.chisq) then
            r = rtemp
            f = ftemp
            d = dtemp
            chisq0 = chitemp
        else
            f = -f
            d = -d
            r = 1/r
        endif

        return
        end


C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 
        SUBROUTINE FIND_MIN_R(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >                                  m1,m2,nmask,r,f,d,dr,rnew,chisq)  

        implicit none

        integer nmax,ntot,i,j,mpoly,n1,n2,max_iter,count,nmask
	integer	m1(nmask),m2(nmask)

        parameter (mpoly=3)    
*                 # of coefficients in fitting polynomial 
        parameter (nmax=1024)                                             
        parameter (max_iter=50)   
*                 maximum iterations 

        real wave(ntot),wave1(nmax)
        real flux_1(ntot),sig_1(ntot)
        real flux_2(ntot),sig_2(ntot)
        real dr,r,f,d,chisq,rnew,ratio
        real work1(nmax),work2(nmax),work3(nmax),work4(nmax)
        real diff(nmax),sigma(nmax)
        real coeff(mpoly), chi(3)                       

        count = 0

C-----------------------------------------------------------------------
C    Put the convolved and shifted spectra into work3 and work4

C-----------------------------------------------------------------------
C   Apply convolution with FWHM = "f"

        call conv_gauss(flux_2,work1,f/2.35482,ntot,.false.)
        call conv_gauss(sig_2,work2,f/2.35482,ntot,.true.)

C-----------------------------------------------------------------------
C   Apply shift by "d" pixels

        call shift_spec(work1,work3,ntot,d,.false.)
        call shift_spec(work2,work4,ntot,d,.true.)

C----------------------------------------------------------------------- 
C   Fill the wavelength work array

        do i = n1,n2
           wave1(i-n1+1) = wave(i)
        enddo

C----------------------------------------------------------------------- 
C   Find chisq at three positions r-dr,r and r+dr

  001   do j = 1,3            
           ratio = r+(j-2)*dr
C----------------------------------------------------------------------- 
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range

           do i = n1,n2
              diff(i-n1+1) = flux_1(i) - ratio*work3(i)
              sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
           enddo

C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq

           call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                          coeff,mpoly,chisq)
           chi(j) = chisq
        enddo

C----------------------------------------------------------------------- 
C   Find minimum in chisq

  002   count = count + 1
D       write(6,98) ratio,chi(1),chi(2),chi(3)
D 098   format(' Scanning R: Ratio = ',f10.5,' Chisq = ',3f12.2)
        if(count.ge.max_iter) then 
           rnew = ratio
           goto 4
        endif
        if (chi(2).le.chi(1)) then
           if (chi(3).lt.chi(2)) then     

C----------------------------------------------------------------------- 
C   Chisq decreases with increasing ratio
              
              chi(1) = chi(2)
              chi(2) = chi(3)
              ratio = ratio+dr
              do i = n1,n2
                 diff(i-n1+1) = flux_1(i) - ratio*work3(i)
                 sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
              enddo
              call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                         coeff,mpoly,chisq)
              chi(3) = chisq
              goto 2
           else

C----------------------------------------------------------------------- 
C   Minimum found !!!
              goto 3
           endif
        else
           if (chi(3).ge.chi(2)) then

C----------------------------------------------------------------------- 
C   Chisq increases with decreasing ratio
              
              chi(3) = chi(2)
              chi(2) = chi(1)
              ratio = ratio - 3*dr
              do i = n1,n2
                 diff(i-n1+1) = flux_1(i) - ratio*work3(i)
                 sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
              enddo
              call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                         coeff,mpoly,chisq)
              chi(1) = chisq    
              ratio = ratio + 2*dr
              goto 2
           else

C----------------------------------------------------------------------- 
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.

              if(chi(3).lt.chi(1)) r = r + dr
              if(chi(3).gt.chi(1)) r = r - dr
              goto 1
           endif
        endif

C----------------------------------------------------------------------- 
C   Calculate the actual position of the minimum in chisq by a parabolic 
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = ratio)

  003   rnew = ratio - dr * (0.5 + (chi(3)-chi(2))/
     >                                      (chi(3)-2*chi(2)+chi(1)))

C----------------------------------------------------------------------- 
C   Calculate the actual minimum chisq

  004   do i = n1,n2
           diff(i-n1+1) = flux_1(i) - rnew*work3(i)
           sigma(i-n1+1) = sqrt(sig_1(i)**2+(rnew*work4(i))**2)
        enddo
        call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                            coeff,mpoly,chisq)
D       write(6,99) rnew, chisq
D 099   format(' Found minimum chisq in R: Ratio = ',f10.5,' Chisq = ',
D    >               f12.2)

        return
        end


C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 
        SUBROUTINE FIND_MIN_F(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >                                 m1,m2,nmask,r,f,d,df,fnew,chisq)  

        implicit none

        integer nmax,ntot,i,j,mpoly,n1,n2,max_iter,count,nmask
	integer	m1(nmask),m2(nmask)

        parameter (mpoly=3)    
*                 # of coefficients in fitting polynomial 
        parameter (nmax=1024)                                             
        parameter (max_iter=50)

        real wave(ntot),wave1(nmax)
        real flux_1(ntot),sig_1(ntot)
        real flux_2(ntot),sig_2(ntot)
        real df,r,f,d,chisq,fnew,FWHM
        real work1(nmax),work2(nmax),work3(nmax),work4(nmax)
        real diff(nmax),sigma(nmax)
        real coeff(mpoly), chi(3)

        count = 0

C-----------------------------------------------------------------------
C    Put the scaled and shifted spectra into work1 and work2

C-----------------------------------------------------------------------
C   Apply shift by "d" pixels

        call shift_spec(flux_2,work1,ntot,d,.false.)
        call shift_spec(sig_2,work2,ntot,d,.true.)

C-----------------------------------------------------------------------
C   Scale spectra by present flux factor

        do i = 1,ntot
           work1(i) = work1(i)*r
           work2(i) = work2(i)*r
        enddo

C----------------------------------------------------------------------- 
C   Fill the wavelength work array

        do i = n1,n2
           wave1(i-n1+1) = wave(i)
        enddo

C----------------------------------------------------------------------- 
C   Find chisq at three positions f,f+df,f+2df

  001   do j = 1,3            
           FWHM = f+(j-1)*df
C----------------------------------------------------------------------- 
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range

           call conv_gauss(work1,work3,FWHM/2.35482,ntot,.false.)
           call conv_gauss(work2,work4,FWHM/2.35482,ntot,.true.)

           do i = n1,n2     
              diff(i-n1+1) = flux_1(i) - work3(i)
              sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
           enddo

C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq

           call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                   coeff,mpoly,chisq)
           chi(j) = chisq
        enddo

C----------------------------------------------------------------------- 
C   Find minimum in chisq

  002   count = count + 1
D       write(6,98) FWHM,chi(1),chi(2),chi(3)
D 098   format(' Scanning F: FWHM  = ',f10.5,' Chisq = ',3f12.2)
        if(count.ge.max_iter) then 
           fnew = FWHM
           goto 4
        endif
        if (chi(2).le.chi(1)) then
           if (chi(3).lt.chi(2)) then     

C----------------------------------------------------------------------- 
C   Chisq decreases with increasing FWHM
              
              chi(1) = chi(2)
              chi(2) = chi(3)
              FWHM = FWHM+df
              call conv_gauss(work1,work3,FWHM/2.35482,ntot,.false.)
              call conv_gauss(work2,work4,FWHM/2.35482,ntot,.true.)
              do i = n1,n2
                 diff(i-n1+1) = flux_1(i) - work3(i)
                 sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
              enddo
              call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                   coeff,mpoly,chisq)
              chi(3) = chisq
              goto 2
           else

C----------------------------------------------------------------------- 
C   Minimum found !!!
              goto 3
           endif
        else
           if (chi(3).ge.chi(2)) then

C----------------------------------------------------------------------- 
C   Chisq increases with decreasing FWHM
              
              chi(3) = chi(2)
              chi(2) = chi(1)
              FWHM = FWHM - 3*df
              call conv_gauss(work1,work3,FWHM/2.35482,ntot,.false.)
              call conv_gauss(work2,work4,FWHM/2.35482,ntot,.true.)
              do i = n1,n2
                 diff(i-n1+1) = flux_1(i) - work3(i)
                 sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
              enddo
              call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                       coeff,mpoly,chisq)
              chi(1) = chisq    
              FWHM = FWHM + 2*df
              goto 2
           else

C----------------------------------------------------------------------- 
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.

              if(chi(3).lt.chi(1)) f = f + df
              if(chi(3).gt.chi(1)) f = f - df
              goto 1
           endif
        endif

C----------------------------------------------------------------------- 
C   Calculate the actual position of the minimum in chisq by a parabolic 
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = FWHM)

  003   fnew = FWHM - df * (0.5 + (chi(3)-chi(2))/
     >                                      (chi(3)-2*chi(2)+chi(1)))

C----------------------------------------------------------------------- 
C   Calculate the actual minimum chisq

  004   call conv_gauss(work1,work3,fnew/2.35482,ntot,.false.)
        call conv_gauss(work2,work4,fnew/2.35482,ntot,.true.)

        do i = n1,n2
           diff(i-n1+1) = flux_1(i) - work3(i)
           sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
        enddo

        call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                     coeff,mpoly,chisq)
D       write(6,99) fnew, chisq
D 099   format(' Found minimum chisq in F: FWHM  = ',f10.5,' Chisq = ',
D    >              f12.2)

        return
        end


C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 
        SUBROUTINE FIND_MIN_D(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     >                               m1,m2,nmask,r,f,d,dd,dnew,chisq)  

        implicit none

        integer nmax,ntot,i,j,mpoly,n1,n2,count,max_iter,nmask
	integer	m1(nmask),m2(nmask)

        parameter (mpoly=3)    
*                 # of coefficients in fitting polynomial 
        parameter (nmax=1024)                           
        parameter (max_iter=50)   
*                 maximum iterations 
          

        real wave(ntot),wave1(nmax)
        real flux_1(ntot),sig_1(ntot)
        real flux_2(ntot),sig_2(ntot)
        real dd,r,f,d,chisq,dnew,shift
        real work1(nmax),work2(nmax),work3(nmax),work4(nmax)
        real diff(nmax),sigma(nmax)
        real coeff(mpoly), chi(3)

        count = 0

C-----------------------------------------------------------------------
C    Put the scaled and convolved spectra into work1 and work2

C-----------------------------------------------------------------------
C   Apply convolution with FWHM = "f"

        call conv_gauss(flux_2,work1,f/2.35482,ntot,.false.)
        call conv_gauss(sig_2,work2,f/2.35482,ntot,.true.)

C-----------------------------------------------------------------------
C   Scale spectra by present flux factor

        do i = 1,ntot
           work1(i) = work1(i)*r
           work2(i) = work2(i)*r
        enddo

C----------------------------------------------------------------------- 
C   Fill the wavelength work array

        do i = n1,n2
           wave1(i-n1+1) = wave(i)
        enddo

C----------------------------------------------------------------------- 
C   Find chisq at three positions d-dd,d and d+dd

  001   do j = 1,3            
           shift = d+(j-2)*dd
C----------------------------------------------------------------------- 
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range

           call shift_spec(work1,work3,ntot,shift,.false.)
           call shift_spec(work2,work4,ntot,shift,.true.)

           do i = n1,n2
              diff(i-n1+1) = flux_1(i) - work3(i)
              sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
           enddo

C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq

           call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                        coeff,mpoly,chisq)
           chi(j) = chisq
        enddo

C----------------------------------------------------------------------- 
C   Find minimum in chisq

  002   count = count + 1
D       write(6,98) shift,chi(1),chi(2),chi(3)
D 098   format(' Scanning D: Shift = ',f10.5,' Chisq = ',3f12.2)
        if(count.ge.max_iter) then 
           dnew = shift
           goto 4
        endif
        if(count.ge.max_iter) goto 3
        if (chi(2).le.chi(1)) then
           if (chi(3).lt.chi(2)) then     

C----------------------------------------------------------------------- 
C   Chisq decreases with increasing ratio
              
              chi(1) = chi(2)
              chi(2) = chi(3)
              shift = shift+dd
              call shift_spec(work1,work3,ntot,shift,.false.)
              call shift_spec(work2,work4,ntot,shift,.true.)
              do i = n1,n2
                 diff(i-n1+1) = flux_1(i) - work3(i)
                 sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
              enddo
              call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                    coeff,mpoly,chisq)
              chi(3) = chisq
              goto 2
           else

C----------------------------------------------------------------------- 
C   Minimum found !!!
              goto 3
           endif
        else
           if (chi(3).ge.chi(2)) then

C----------------------------------------------------------------------- 
C   Chisq increases with increasing shift
              
              chi(3) = chi(2)
              chi(2) = chi(1)
              shift = shift - 3*dd
              call shift_spec(work1,work3,ntot,shift,.false.)
              call shift_spec(work2,work4,ntot,shift,.true.)
              do i = n1,n2
                 diff(i-n1+1) = flux_1(i) - work3(i)
                 sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
              enddo
              call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                     coeff,mpoly,chisq)
              chi(1) = chisq    
              shift = shift + 2*dd
              goto 2
           else

C----------------------------------------------------------------------- 
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.

              if(chi(3).lt.chi(1)) d = d + dd
              if(chi(3).gt.chi(1)) d = d - dd
              goto 1
           endif
        endif

C----------------------------------------------------------------------- 
C   Calculate the actual position of the minimum in chisq by a parabolic 
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = shift)

  003   dnew = shift - dd * (0.5 + (chi(3)-chi(2))/
     >                                      (chi(3)-2*chi(2)+chi(1)))

C----------------------------------------------------------------------- 
C   Calculate the actual minimum chisq

  004   call shift_spec(work1,work3,ntot,dnew,.false.)
        call shift_spec(work2,work4,ntot,dnew,.true.)
        do i = n1,n2
           diff(i-n1+1) = flux_1(i) - work3(i)
           sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
        enddo
        call fitpoly(wave1,diff,sigma,n2-n1+1,m1,m2,nmask,
     &                                         coeff,mpoly,chisq)
D       write(6,99) dnew, chisq
D 099   format(' Found minimum chisq in D: Shift = ',f10.5,' Chisq = ',
D    >              f12.2)

        return
        end


C----------------------------------------------------------------------- 
C----------------------------------------------------------------------- 
C    Output the position of the minimum and the value of chisq to terminal

        subroutine write_res(r,f,d,chisq)

        implicit none
        save             

        real r,f,d,chisq

        write(6,1) r,f,d,chisq
  001   format(' Minimum found: Ratio =',f8.3,'   FWHM =',f6.2,
     >        '  Shift =',f6.2,'   Chisq =',f8.2)
        
	open(unit=38, file='results',status='unknown',
     &      access='sequential',fileopt='eof')
        write(38,2) r,f,d,chisq
  002   format(f9.4,f7.3,f7.3,f9.3)
        close(unit=38)

        return
        end

C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
        SUBROUTINE FITPOLY(x,y,sigma,n,m1,m2,nmask,coeff,mpoly,chisq)

        integer mmax,n,mpoly,i,j,nmax,nmask
	integer m1(nmask),m2(nmask)
	parameter (nmax=1024)
*	          maximum expected number of pixels in the fitting region
        parameter (mmax=10)        
*                 maximum order of fitting polynomial +1 
        integer lista(mmax), nelm
        real covar (mmax,mmax), chisq
        real x(1),y(1),sigma(1),coeff(1)
        real xc, a(nmax), b(nmax), c(nmax)
	logical do_fit
        
C-----------------------------------------------------------------------
C    Initialize array LISTA 

        do i = 1,mpoly
            lista(i) = i
        enddo

C-----------------------------------------------------------------------
C    Center x-range to zero (otherwise LFIT may yield incorrect results)
       
        xc = (x(n)+x(1))/2.0
        do i = 1,n
           x(i) = x(i)-xc
        enddo

* use dummy arrays to take out the masked regions that should not be fitted

	i = 1
	k = 1
	do j = 1, n
	   do_fit = .true.
	   do while (do_fit.and.k.le.nmask)
	      if (j.gt.m1(k).and.j.lt.m2(k)) do_fit = .false.
	      k = k + 1
	   enddo
	   if (do_fit) then
	      a(i) = x(j)
	      b(i) = y(j)
	      c(i) = sigma(j)
	      i = i + 1
	   endif
	enddo
	nelm = i - 1

C-----------------------------------------------------------------------

        call lfit(a,b,c,nelm,coeff,mpoly,lista,mpoly,covar,mpoly,chisq)

C-----------------------------------------------------------------------
C    Restore original x--range
       
        do i = 1,n
           x(i) = x(i)+xc
        enddo

        return
        end

C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
        SUBROUTINE SHIFT_SPEC(input,output,n,shift,sigma)

C   Shifts the input spectrum by "shift" pixels into the output spectrum
C   Shift not necessarily integer number
*   Note that a shift in a flux array is not the same as a shift in an
*   error array. In the latter case we need to propagate the errors of
*   the points used for the interpolation.

        implicit         none 
        integer          n,i,round
        real             input(1),output(1),shift
        real             sign1,frac
        logical          sigma

        sign1 = sign(1.0,shift)
        round = int(abs(shift)) 
*               truncation; gives # pixels to hop over 
        frac  = abs(shift) - round
        
        if (.not.sigma) then  
*                      flux shifting
           if (sign1.gt.0) then
              do i = 1,n-round-1                
               output(i) = (1.0-frac)*input(i+round) + 
     &                                      frac*input(i+round+1)
              enddo 
              do i = n-round,n
                output(i) = input(i)
              enddo
           else
              do i = round+2,n
               output(i) = (1.0-frac)*input(i-round) + 
     &                                      frac*input(i-round-1)
              enddo
              do i = 1,round+1
                output(i) = input(i)
              enddo
           endif
        else          
*          sigma shifting
           if (sign1.gt.0) then
              do i = 1,n-round-1                
               output(i) =sqrt( ((1.0-frac)*input(i+round))**2 +
     &                          (frac*input(i+round+1))**2 )
              enddo 
              do i = n-round,n
                output(i) = input(i)
              enddo
           else
              do i = round+2,n
               output(i) =sqrt( ((1.0-frac)*input(i-round))**2 +
     &                          (frac*input(i-round-1))**2 )
              enddo
              do i = 1,round+1
                output(i) = input(i)
              enddo
           endif
        endif
        
        return
        end


C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
	SUBROUTINE CONV_GAUSS(in,out,sigma,n,flag)

        implicit        none
        real            in(n),out(n),gauss(1000),res(1000)
        real            sum,sigma
        integer         n,nelm2,i,j,nc,nout
        logical         flag

C    Subroutine to convolve array "in" with a gaussian with width "sigma"
*    In case the flag is .true. this means we are convolving an error array
*    and we need to square everything before convolving and to take the 
*    square root after convolution.
C    

C ------------------------------------------------------------------------
        if (sigma.eq.0.0) then
            do i = 1,n
                out(i) = in(i)
            enddo
            return
        endif
        
        nc = max( int(0.5*sigma), 5)
        nelm2 = 2*nc-1

        sum = 0.0
        do i = 1,nelm2 
            gauss(i) = exp(-0.5*((i-nc)/sigma)**2)
            sum = sum + gauss(i)
        enddo                   
        do i = 1, nelm2
            gauss(i) = gauss(i)/sum
        enddo                      

        nout = n - nelm2 + 1 

C ------------------------------------------------------------------------
C       Do the convolution
             
        if (.not. flag) then     
*                      flux convolution 
           do i = 1,nout
               sum = 0.0
               do j = 1,nelm2
                   sum = sum + in(i+j-1) * gauss(nelm2-j+1)
               enddo
               res(i) = sum
           enddo 
        else                     
*          sigma convolution 
           do i = 1,nout
               sum = 0.0
               do j = 1,nelm2
                   sum = sum + in(i+j-1)**2 * gauss(nelm2-j+1)**2
               enddo
               res(i) = sqrt(sum)
           enddo 
        endif

        do i = 1,(nelm2-1)/2
             out(i) = in(i)
        enddo

        do i = 1,nout
             out((nelm2-1)/2+i) = res(i)
        enddo 

        do i = nout+1,n
             out(i) = in(i)
        enddo

        end


C ------------------------------------------------------------------------
C
C  ALL FOLLOWING ROUTINES WERE TAKEN FROM Press et al., Numerical Recipes.
C
C ------------------------------------------------------------------------
      SUBROUTINE LFIT(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,COVAR,NCVM,CHISQ)
      PARAMETER (MMAX=50)
      DIMENSION X(NDATA),Y(NDATA),SIG(NDATA),A(MA),LISTA(MFIT),
     *    COVAR(NCVM,NCVM),BETA(MMAX),AFUNC(MMAX)
      KK=MFIT+1
      DO 12 J=1,MA
        IHIT=0
        DO 11 K=1,MFIT
          IF (LISTA(K).EQ.J) IHIT=IHIT+1
11      CONTINUE
        IF (IHIT.EQ.0) THEN
          LISTA(KK)=J
          KK=KK+1
        ELSE IF (IHIT.GT.1) THEN
          PAUSE 'Improper set in LISTA'
        ENDIF
12    CONTINUE
      IF (KK.NE.(MA+1)) PAUSE 'Improper set in LISTA'
      DO 14 J=1,MFIT
        DO 13 K=1,MFIT
          COVAR(J,K)=0.
13      CONTINUE
        BETA(J)=0.
14    CONTINUE
      DO 18 I=1,NDATA
        CALL FUNCS(X(I),AFUNC,MA)
        YM=Y(I)
        IF(MFIT.LT.MA) THEN
          DO 15 J=MFIT+1,MA
            YM=YM-A(LISTA(J))*AFUNC(LISTA(J))
15        CONTINUE
        ENDIF
        SIG2I=1./SIG(I)**2
        DO 17 J=1,MFIT
          WT=AFUNC(LISTA(J))*SIG2I
          DO 16 K=1,J
            COVAR(J,K)=COVAR(J,K)+WT*AFUNC(LISTA(K))
16        CONTINUE
          BETA(J)=BETA(J)+YM*WT
17      CONTINUE
18    CONTINUE
      IF (MFIT.GT.1) THEN
        DO 21 J=2,MFIT
          DO 19 K=1,J-1
            COVAR(K,J)=COVAR(J,K)
19        CONTINUE
21      CONTINUE
      ENDIF
      CALL GAUSSJ(COVAR,MFIT,NCVM,BETA,1,1)
      DO 22 J=1,MFIT
        A(LISTA(J))=BETA(J)
22    CONTINUE
      CHISQ=0.
      DO 24 I=1,NDATA
        CALL FUNCS(X(I),AFUNC,MA)
        SUM=0.
        DO 23 J=1,MA
          SUM=SUM+A(J)*AFUNC(J)
23      CONTINUE
        CHISQ=CHISQ+((Y(I)-SUM)/SIG(I))**2
24    CONTINUE
      CALL COVSRT(COVAR,NCVM,MA,LISTA,MFIT)
      RETURN
      END
      SUBROUTINE COVSRT(COVAR,NCVM,MA,LISTA,MFIT)
      DIMENSION COVAR(NCVM,NCVM),LISTA(MFIT)
      DO 12 J=1,MA-1
        DO 11 I=J+1,MA
          COVAR(I,J)=0.
11      CONTINUE
12    CONTINUE
      DO 14 I=1,MFIT-1
        DO 13 J=I+1,MFIT
          IF(LISTA(J).GT.LISTA(I)) THEN
            COVAR(LISTA(J),LISTA(I))=COVAR(I,J)
          ELSE
            COVAR(LISTA(I),LISTA(J))=COVAR(I,J)
          ENDIF
13      CONTINUE
14    CONTINUE
      SWAP=COVAR(1,1)
      DO 15 J=1,MA
        COVAR(1,J)=COVAR(J,J)
        COVAR(J,J)=0.
15    CONTINUE
      COVAR(LISTA(1),LISTA(1))=SWAP
      DO 16 J=2,MFIT
        COVAR(LISTA(J),LISTA(J))=COVAR(1,J)
16    CONTINUE
      DO 18 J=2,MA
        DO 17 I=1,J-1
          COVAR(I,J)=COVAR(J,I)
17      CONTINUE
18    CONTINUE
      RETURN
      END
      SUBROUTINE GAUSSJ(A,N,NP,B,M,MP)
      PARAMETER (NMAX=50)
      DIMENSION A(NP,NP),B(NP,MP),IPIV(NMAX),INDXR(NMAX),INDXC(NMAX)
      DO 11 J=1,N
        IPIV(J)=0
11    CONTINUE
      DO 22 I=1,N
        BIG=0.
        DO 13 J=1,N
          IF(IPIV(J).NE.1)THEN
            DO 12 K=1,N
              IF (IPIV(K).EQ.0) THEN
                IF (ABS(A(J,K)).GE.BIG)THEN
                  BIG=ABS(A(J,K))
                  IROW=J
                  ICOL=K
                ENDIF
              ELSE IF (IPIV(K).GT.1) THEN
		print*,' *** FATAL *** A singular matrix occurred!'
	        print*,' Is this because in both spectra the background'
		print*,' is (virtually) zero? If so, you can solve'
		print*,' this error by adding a constant to one of the'
		print*,' input spectra before running this program'
		print*,' and subtract the constant afterwards ***'
                stop ' *** PROGRAM ABORTED ***'
              ENDIF
12          CONTINUE
          ENDIF
13      CONTINUE
        IPIV(ICOL)=IPIV(ICOL)+1
        IF (IROW.NE.ICOL) THEN
          DO 14 L=1,N
            DUM=A(IROW,L)
            A(IROW,L)=A(ICOL,L)
            A(ICOL,L)=DUM
14        CONTINUE
          DO 15 L=1,M
            DUM=B(IROW,L)
            B(IROW,L)=B(ICOL,L)
            B(ICOL,L)=DUM
15        CONTINUE
        ENDIF
        INDXR(I)=IROW
        INDXC(I)=ICOL
        IF (A(ICOL,ICOL).EQ.0.) PAUSE 'Singular matrix.'
        PIVINV=1./A(ICOL,ICOL)
        A(ICOL,ICOL)=1.
        DO 16 L=1,N
          A(ICOL,L)=A(ICOL,L)*PIVINV
16      CONTINUE
        DO 17 L=1,M
          B(ICOL,L)=B(ICOL,L)*PIVINV
17      CONTINUE
        DO 21 LL=1,N
          IF(LL.NE.ICOL)THEN
            DUM=A(LL,ICOL)
            A(LL,ICOL)=0.
            DO 18 L=1,N
              A(LL,L)=A(LL,L)-A(ICOL,L)*DUM
18          CONTINUE
            DO 19 L=1,M
              B(LL,L)=B(LL,L)-B(ICOL,L)*DUM
19          CONTINUE
          ENDIF
21      CONTINUE
22    CONTINUE
      DO 24 L=N,1,-1
        IF(INDXR(L).NE.INDXC(L))THEN
          DO 23 K=1,N
            DUM=A(K,INDXR(L))
            A(K,INDXR(L))=A(K,INDXC(L))
            A(K,INDXC(L))=DUM
23        CONTINUE
        ENDIF
24    CONTINUE
      RETURN
      END


        SUBROUTINE FUNCS(X,P,NP)
        
        implicit none
                      
        integer np ,i
        real p(np),x

        p(1) = 1.0
        do i = 2,np
            p(i) = p(i-1)*x
        enddo
        return
        end

c-----------------------------------------------EOF


