
C---------------------------------------------------------------------------

      subroutine find_min_D(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                      r,f,d,dd,dnew,chisq)
 
C     implicit none
 
      integer NMAX,ntot,i,j,mpoly,n1,n2,count,max_iter
      
C  # of coefficients in fitting polynomial
      parameter (MPOLY=3)
      parameter (NMAX=1024)
C  maximum iterations
      parameter (MAX_ITER=50)
 
 
      real wave(ntot),wave1(NMAX)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real dd,r,f,d,chisq,dnew,shift
      real work1(NMAX),work2(NMAX),work3(NMAX),work4(NMAX)
      real diff(NMAX),sigma(NMAX)
      real coeff(MPOLY), chi(3)
 
      count = 0
 
C-----------------------------------------------------------------------
C    Put the scaled and convolved spectra into work1 and work2
      
C-----------------------------------------------------------------------
C   Apply convolution with FWHM = "f"
 
      call conv_gauss(flux_2,work1,f/2.35482,ntot,.false.)
      call conv_gauss(sig_2,work2,f/2.35482,ntot,.true.)
 
C-----------------------------------------------------------------------
C   Scale spectra by present flux factor
 
      do 100 i = 1,ntot
         work1(i) = work1(i)*r
         work2(i) = work2(i)*r
 100  continue
 
C-----------------------------------------------------------------------
C   Fill the wavelength work array
 
      do 110 i = n1,n2
         wave1(i-n1+1) = wave(i)
 110  continue
 
C-----------------------------------------------------------------------
C   Find chisq at three positions d-dd,d and d+dd
 
 1    do 200 j = 1,3
         shift = d + float(j-2)*dd
C-----------------------------------------------------------------------
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range
 
         call shift_spec(work1,work3,ntot,shift,.false.)
         call shift_spec(work2,work4,ntot,shift,.true.)
 
         do 210 i = n1,n2
            diff(i-n1+1) = flux_1(i) - work3(i)
            sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 210     continue
 
C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq
 
         call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
         chi(j) = chisq
 200  continue
 
C-----------------------------------------------------------------------
C   Find minimum in chisq
 
 2    count = count + 1

C*      write(6,98) shift,chi(1),chi(2),chi(3)
C*98   format(' Scanning D: Shift = ',f10.5,' Chisq = ',3f12.2)

      if (count .ge. MAX_ITER) then
         dnew = shift
         goto 4
      end if
      if (count .ge. MAX_ITER) goto 3
      if (chi(2) .le. chi(1)) then
         if (chi(3) .lt. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq decreases with increasing ratio
 
            chi(1) = chi(2)
            chi(2) = chi(3)
            shift = shift+dd
            call shift_spec(work1,work3,ntot,shift,.false.)
            call shift_spec(work2,work4,ntot,shift,.true.)
            do 300 i = n1,n2
               diff(i-n1+1) = flux_1(i) - work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 300        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(3) = chisq
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Minimum found !!!

            goto 3
         end if
      else
         if (chi(3) .ge. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq increases with increasing shift
 
            chi(3) = chi(2)
            chi(2) = chi(1)
            shift = shift - 3.0*dd
            call shift_spec(work1,work3,ntot,shift,.false.)
            call shift_spec(work2,work4,ntot,shift,.true.)
            do 400 i = n1,n2
               diff(i-n1+1) = flux_1(i) - work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 400        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(1) = chisq
            shift = shift + 2.0*dd
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.
 
            if (chi(3) .lt. chi(1)) d = d + dd
            if (chi(3) .gt. chi(1)) d = d - dd
            goto 1
         end if
      end if
 
C-----------------------------------------------------------------------
C   Calculate the actual position of the minimum in chisq by a parabolic
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = shift)
 
 3    dnew = shift - dd * (0.5 + (chi(3)-chi(2))/
     &       (chi(3)-2.0*chi(2)+chi(1)))
 
C-----------------------------------------------------------------------
C   Calculate the actual minimum chisq
 
 4    call shift_spec(work1,work3,ntot,dnew,.false.)
      call shift_spec(work2,work4,ntot,dnew,.true.)
      do 500 i = n1,n2
         diff(i-n1+1) = flux_1(i) - work3(i)
         sigma(i-n1+1) = sqrt(sig_1(i)**2+work4(i)**2)
 500  continue
      call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)

C*       write(6,99) dnew, chisq
C* 099   format(' Found minimum chisq in D: Shift = ',f10.5,' Chisq = ',
C*    >              f12.2)
 
      return
      end
