
C---------------------------------------------------------------------------

      subroutine find_min_r(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                      r,f,d,dr,rnew,chisq)
 
c     implicit none
 
      integer NMAX,ntot,i,j,MPOLY,n1,n2,MAX_ITER,count

C  # of coefficients in fitting polynomial
      parameter (MPOLY=3)
      parameter (NMAX=1024)
C  maximum iterations
      parameter (MAX_ITER=50)
 
      real wave(ntot),wave1(NMAX)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real dr,r,f,d,chisq,rnew,ratio
      real work1(NMAX),work2(NMAX),work3(NMAX),work4(NMAX)
      real diff(NMAX),sigma(NMAX)
      real coeff(MPOLY), chi(3)
 
      count = 0
 
C-----------------------------------------------------------------------
C    Put the convolved and shifted spectra into work3 and work4
 
C-----------------------------------------------------------------------
C   Apply convolution with FWHM = "f"
 
      call conv_gauss(flux_2,work1,f/2.35482,ntot,.false.)
      call conv_gauss(sig_2,work2,f/2.35482,ntot,.true.)
 
C-----------------------------------------------------------------------
C   Apply shift by "d" pixels
 
      call shift_spec(work1,work3,ntot,d,.false.)
      call shift_spec(work2,work4,ntot,d,.true.)
 
C-----------------------------------------------------------------------
C   Fill the wavelength work array
 
      do 100 i = n1, n2
         wave1(i-n1+1) = wave(i)
 100  continue
 
C-----------------------------------------------------------------------
C   Find chisq at three positions r-dr,r and r+dr
 
 1    do 200 j = 1,3
         ratio = r + float(j-2)*dr

C-----------------------------------------------------------------------
C   Fill the arrays wave1,diff and sigma with wavelengths, difference
C   and sigma's resp. for the required wavelength range
 
         do 210 i = n1,n2
            diff(i-n1+1) = flux_1(i) - ratio*work3(i)
            sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
 210     continue
 
C-----------------------------------------------------------------------
C   Do the polynomial fitting and calculate chisq
 
         call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
         chi(j) = chisq
 200  continue
 
C-----------------------------------------------------------------------
C   Find minimum in chisq
 
 2    count = count + 1
C*       write(6,98) ratio,chi(1),chi(2),chi(3)
C* 98    format(' Scanning R: Ratio = ',f10.5,' Chisq = ',3f12.2)
      if (count .ge. MAX_ITER) then
         rnew = ratio
         goto 4
      end if
      if (chi(2) .le. chi(1)) then
         if (chi(3) .lt. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq decreases with increasing ratio
 
            chi(1) = chi(2)
            chi(2) = chi(3)
            ratio = ratio+dr
            do 300 i = n1,n2
               diff(i-n1+1) = flux_1(i) - ratio*work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
 300        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(3) = chisq
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Minimum found !!!

            goto 3
         end if
      else
         if (chi(3) .ge. chi(2)) then
 
C-----------------------------------------------------------------------
C   Chisq increases with decreasing ratio
 
            chi(3) = chi(2)
            chi(2) = chi(1)
            ratio = ratio - 3.0*dr
            do 400 i = n1,n2
               diff(i-n1+1) = flux_1(i) - ratio*work3(i)
               sigma(i-n1+1) = sqrt(sig_1(i)**2+(ratio*work4(i))**2)
 400        continue
            call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)
            chi(1) = chisq
            ratio = ratio + 2.0*dr
            goto 2
         else
 
C-----------------------------------------------------------------------
C   Oops, local maximum in chisq. Choose the direction of the lowest and
C      start from the beginning.
 
            if (chi(3) .lt. chi(1)) r = r + dr
            if (chi(3) .gt. chi(1)) r = r - dr
            goto 1
         end if
      end if
 
C-----------------------------------------------------------------------
C   Calculate the actual position of the minimum in chisq by a parabolic
C   fit to the last three values in chi (Bevington, p210).
C   (Where a(3) = ratio)
 
 3    rnew = ratio - dr * (0.5 + (chi(3)-chi(2))/
     &                    (chi(3)-2.0*chi(2)+chi(1)))
 
C-----------------------------------------------------------------------
C   Calculate the actual minimum chisq
 
 4    do 500 i = n1,n2
         diff(i-n1+1) = flux_1(i) - rnew*work3(i)
         sigma(i-n1+1) = sqrt(sig_1(i)**2+(rnew*work4(i))**2)
 500  continue
      call fitpoly(wave1,diff,sigma,n2-n1+1,coeff,MPOLY,chisq)

C*       write(6,99) rnew, chisq
C* 99    format(' Found minimum chisq in R: Ratio = ',f10.5,' Chisq = ',
C*    >               f12.2)
 
      return
      end
