      
C---------------------------------------------------------------------------

      subroutine find_min(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                    r0,f0,d0,dr,df,dd,halt,r,f,d,chisq0,stat)
 
C  Declarations
 
      integer n1,n2,ntot,count,stat
      real wave(ntot)
      real flux_1(ntot),sig_1(ntot)
      real flux_2(ntot),sig_2(ntot)
      real r0,f0,d0,dr,df,dd,r,f,d,halt,chisq
      real rnew,fnew,dnew,chisq0
      real rtemp,ftemp,dtemp,chitemp
      logical proceed
 
C  Maximum number of iterations

      integer MAX_ITER
      parameter (MAX_ITER=30)
 
C  Minimum found if stat not changed

      stat = 0
      count = 0
 
      proceed = .true.
      r = r0
      f = f0
      d = d0
 
      chisq0 = 1.0e6
 
C-----------------------------------------------------------------------
C   Start the grid search
 
      do while (proceed)
         count = count + 1
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying r and constant f and d.
 
         call find_min_r(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &                   r,f,d,dr,rnew,chisq)
         r = rnew
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying d and constant r and f.
 
         call find_min_d(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &        r,f,d,dd,dnew,chisq)
         d = dnew
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying f and constant r and d.
 
         call find_min_f(wave,flux_1,sig_1,flux_2,sig_2,ntot,n1,n2,
     &        r,f,d,df,fnew,chisq)
         f = fnew
 
C-----------------------------------------------------------------------
 
         if (abs(chisq-chisq0) .lt. halt*ntot) proceed = .false.
         if (count .ge. MAX_ITER) then
            proceed = .false.
            stat = 1
         end if
         chisq0 = chisq
      end do
 
C-----------------------------------------------------------------------
C   Store the values of r, f, d and chisq and interchange the files
C   to check if a lower minimum is found when the other file is
C   convolved
 
      rtemp = r
      ftemp = f
      dtemp = d
      chitemp = chisq0
 
C-----------------------------------------------------------------------
C   Repeat procedure with the files interchanged
 
      count = 0
 
      proceed = .true.
      r = r0
      f = f0
      d = d0
 
      chisq0 = 1.0e6
 
C-----------------------------------------------------------------------
C   Start the grid search
 
      do while (proceed)
         count = count + 1
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying r and constant f and d.
 
         call find_min_r(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     &        r,f,d,dr,rnew,chisq)
         r = rnew
            
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying d and constant r and f.
 
         call find_min_d(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     &        r,f,d,dd,dnew,chisq)
         d = dnew
 
C-----------------------------------------------------------------------
C   Find minimum in chisq by varying f and constant r and d.
 
         call find_min_f(wave,flux_2,sig_2,flux_1,sig_1,ntot,n1,n2,
     &        r,f,d,df,fnew,chisq)
         f = fnew
 
C-----------------------------------------------------------------------
 
         if (abs(chisq-chisq0) .lt. halt*ntot) then
            proceed = .false.
            stat = 0
         end if
         if (count .ge. MAX_ITER) then
            proceed = .false.
            print *,'WARNING: FIND_MIN did not converge in 2nd pass'
         end if
         chisq0 = chisq
      end do
 
      if (chitemp .lt. chisq) then
         r = rtemp
         f = ftemp
         d = dtemp
         chisq0 = chitemp
      else
         f = -f
         d = -d
         r = 1.0/r
      end if
 
      return
      end
