
      program mkdiff

C
C  MKDIFF - Interactive program to create a DIFFSPEC configuration file
C
C  Prompts the user interactively for the DIFFSPEC fitting parameters
C  and then loads them into a DIFFSPEC configuration file use for
C  batch or repetitious interactive execution of the DIFFSPEC program
C
C  R. Pogge
C  OSU Astronomy Dept.
C  1993 Feb 26
C
C  Modification History:
C     1993 Feb 26 - new program, adaptations of read_init2 and read_lamb2
C
C--------------------------------------------------------------------------- 

      real wave1, wave2
      real r0, f0, d0, dr, df, dd, halt
      
      character*40 confile
      character*80 errmsg
      integer io_lenc

      logical T
      logical F

C  Yow!

      T = .true.
      F = .false.

      call io_getstr(confile,'diffspec.conf',
     &     'DIFFSPEC configuration file name',ierr)

      open (unit=1, file=confile, status='unknown', iostat=ierr)
      lc = io_lenc(confile)
      if (ierr .ne. 0) then
         errmsg = 'Cannot open the configuration file '//confile(1:lc)
         call io_errmsg(ierr,2,errmsg,T)
      end if

C  Get the parameters from the user

      call io_getreal(wave1,0.,0.,0.,
     &     'Fit Region Lower Wavelength Limit',ierr)

      call io_getreal(wave2,0.,0.,0.,
     &     'Fit Region Upper Wavelength Limit',ierr)
      
      call io_getreal(r0,0.,0.,1.,'Initial Flux Ratio',ierr)
      call io_getreal(f0,0.,0.,1.,'Initial Smoothing Width',ierr)
      call io_getreal(d0,-1.,-1.,0.,'Initial Wavelength Shift',ierr)
      call io_getreal(dr,0.,0.,.01,'Flux Ratio Step Size',ierr)
      call io_getreal(df,0.,0.,.01,'Smoothing Width Step Size',ierr)
      call io_getreal(dd,0.,0.,.001,'Wavelength Shift Step Size',ierr)
      call io_getreal(halt,0.,0.,.00001,'Convergence Criterion',ierr)

C  Load the parameters into the configuration file
 
      write (1,*,iostat=ierr) wave1, wave2
      write (1,*,iostat=ierr) r0, dr
      write (1,*,iostat=ierr) f0, df
      write (1,*,iostat=ierr) d0, dd
      write (1,*,iostat=ierr) halt

      if (ierr .ne. 0) then
         errmsg = 'Cannot write to '//confile(1:lc)
         call io_errmsg(ierr,2,errmsg,F)
      end if

C  Close the file and exit

      close(1)
      print *,'*** MKDIFF Done ***'
      end
