
C---------------------------------------------------------------------------
C
C  READ_SPEC = Modified READ_DATA Subroutine
C
C  Modified at OSU to use different (defaulting) Keyboard I/O, and
C  to autoscale spectra fluxes & sigmas to of order unity to avoid
C  overflow problems in the LSF program.  This version differs
C  from RD_SPEC2 in that it can also read in the file names without
C  prompting.
C
 
      subroutine read_spec (ofile,rfile,wave,flux_1,sigma_1,
     &                      flux_2,sigma_2,fsc,n,havefiles)
 
C  External Variables

      character*(*) ofile
      character*(*) rfile
      integer n
      real wave(n), flux_1(n), flux_2(n)
      real sigma_1(n), sigma_2(n)
      real fsc
      logical havefiles
      
C  Internal Variables

      integer i, n1, n2
      character teststr*40
      logical exist

C  Maximum number of pixels in a spectrum

      integer NMAX
      parameter (NMAX=1024)
      real wave_1(NMAX), wave_2(NMAX)

C   Load the data from the reference spectrum file
 
 666  exist = .false.
 
      if (.not. havefiles) then
         do while (.not.exist)
            call io_getstr(rfile,rfile,
     &           'Name of the Reference Spectrum File',ierr)
            call io_upper(rfile,teststr)
            if (teststr .eq. 'ABORT') stop
            inquire (file=rfile,exist=exist)
            if (.not.exist) then
               write (5,*) ' '
               write (5,*) ' *** WARNING ***  File not found '
               write (5,*) ' Enter ABORT to quit'
               write (5,*) ' '
               rfile = ' '
            end if
         end do
      end if

      open(40, file=rfile, status='old',iostat=ierr)
      if (ierr .ne. 0) then
         call io_errmsg(ierr,2,
     &        'Cannot Open Reference Spectrum File',havefiles)
         go to 666
      end if

      do 100 i = 1, NMAX
         read(40,*,iostat=ierr) wave_1(i),flux_1(i),sigma_1(i)
         if (ierr .lt. 0) go to 1
         if (ierr .ne. 0) call io_errmsg(ierr,2,
     &           'Cannot Read Reference Spectrum File',.true.)
 100  continue
 1    n1 = i-1
      close(40)
 
C   Load the data from the object spectrum file
 
 667  exist = .false.
      if (.not. havefiles) then
         do while (.not.exist)
            call io_getstr(ofile,ofile,
     &           'Name of the Object Spectrum File',ierr)
            call io_upper(ofile,teststr)
            if (teststr .eq. 'ABORT') stop
            inquire (file=ofile,exist=exist)
            if (.not.exist) then
               write (5,*) ' '
               write (5,*) ' *** WARNING ***  File not found '
               write (5,*) ' Enter ABORT to quit'
               write (5,*) ' '
               ofile = ' '
            end if
         end do
      end if

      open(40, file=ofile, status='old', iostat=ierr)
      if (ierr .ne. 0) then
         call io_errmsg(ierr,2,
     &        'Cannot Open Object Spectrum File',havefiles)
         go to 667
      end if
      do 200 i = 1,NMAX
         read(40,*,iostat=ierr) wave_2(i),flux_2(i),sigma_2(i)
         if (ierr .lt. 0) go to 2
         if (ierr .ne. 0) call io_errmsg(ierr,2,
     &           'Cannot Read Object Spectrum File',.true.)
 200  continue
 2    n2 = i-1
      close(40)
 
      if (n1 .ne. n2) call io_errmsg(0,0,
     &     'Spectrum files have different lengths!',.true.)
 
      n = n1
      do 300 i = 1, n
         wave(i) = wave_1(i)
         if (wave_1(i) .ne. wave_2(i)) call io_errmsg(0,0,
     &        'Files have different wavelength scales!',.true.)
 300  continue

C  Autoscale the fluxes to of order unity based on the mean exponent of
C  the reference spectrum.   Remember the flux scaling factor for later
C  to rescale the output fluxes (wrout2.f) to the original flux scale.

      fsc = 0.0
      npts = 0
      do 400 i = 1, n
         farg = abs(flux_1(i))
         if (farg .ne. 0.0) then
            fsc = fsc + float(nint(log10(farg)))
            npts = npts + 1
         end if
 400  continue
      fsc = fsc / float(npts)
      if (abs(fsc) .lt. 3.) then
         fsc = 1.0
         return
      else
         ifsc = nint(fsc) - 1
         fsc = 10.0**(-ifsc)
      end if
      do 401 i = 1, n
         flux_1(i) = fsc*flux_1(i)
         sigma_1(i) = fsc*sigma_1(i)
         flux_2(i) = fsc*flux_2(i)
         sigma_2(i) = fsc*sigma_2(i)
 401  continue

      return
      end
