
C---------------------------------------------------------------------------
C
C  WROUT2 - write the resulting difference spectrum, with flux scaling
C  
C  Modified version of WROUT.F which takes care of the input autoscaling
C  performed on the reference and object spectra by RD_DATA2 and also
C  to write the difference spectrum results in a uniform format favored
C  at OSU
C
C  R. Pogge
C  OSU Astronomy Dept
C  1992 May 20
C

      subroutine wrout2(wave,flux_1,sig_1,flux_2,sig_2,ntot,r,f,d,fsc)
 
      character*40     outfile
      character*40     teststr
      logical          exist
      real wave(ntot)
      real flux_1(ntot), sig_1(ntot)
      real flux_2(ntot), sig_2(ntot)
      real r, f, d, fsc
      integer ntot,i,NMAX

C   Maximum spectral length 

      parameter (NMAX=1024)
      real work1(NMAX), work2(NMAX), work3(NMAX)
      real work4(NMAX), work5(NMAX), work6(NMAX)
 
C-----------------------------------------------------------------------
C   Prompt for filename of output file
 
      exist = .true.
 
      do while (exist)
         call io_getstr(outfile,outfile,
     &        'Output file for the difference spectrum',ierr)
         call io_upper(outfile,teststr)
         if (teststr .eq. 'ABORT') stop
         inquire (file=outfile,exist=exist)
         if (exist) then
            write (5,*) ' '
            write (5,*) ' *** WARNING ***  Already exists '
            write (5,*) ' Enter ABORT to quit'
            write (5,*) ' '
            outfile = ' ' 
         end if
      end do
 
C-----------------------------------------------------------------------
C   Shift the object spectrum by the best-fit shift, 'D' pixels
C   For now:
C   Array WORK1 has the shifted object spectrum fluxes
C   Array WORK2 has the shifted object spectrum sigmas
C
 
      call shift_spec(flux_2,work1,ntot,d,.false.)
      call shift_spec(sig_2,work2,ntot,d,.true.)
 
C-----------------------------------------------------------------------
C   Scale the object spectrum by the best-fit scaling factor, 'R'

      do 100 i = 1, ntot
         work1(i) = r*work1(i)
         work2(i) = r*work2(i)
 100  continue
 
C-----------------------------------------------------------------------
 
C   Smooth the (Reference|Object) spectrum with a Gaussian of FWHM=F 
C   depending whether F is (Negative|Positve)
C
C   Array WORK3 will hold the (smoothed) Object spectrum fluxes
C   Array WORK4 will hold the (smoothed) Object spectrum sigmas
C   Array WORK5 will hold the (smoothed) Reference spectrum fluxes
C   Array WORK6 will hold the (smoothed) Reference spectrum sigmas
C
C   Remove the flux autoscaling at this time, too
C

      if (f .ge. 0.0) then
         call conv_gauss(work1,work3,f/2.35482,ntot,.false.)
         call conv_gauss(work2,work4,f/2.35482,ntot,.true.)
         do 200 i = 1, ntot
            work3(i) = work3(i)/fsc
            work4(i) = work4(i)/fsc
            work5(i) = flux_1(i)/fsc
            work6(i) = sig_1(i)/fsc
 200     continue
      else
         call conv_gauss(flux_1,work5,f/2.35482,ntot,.false.)
         call conv_gauss(sig_1,work6,f/2.35482,ntot,.true.)
         do 201 i = 1, ntot
            work3(i) = work1(i)/fsc
            work4(i) = work2(i)/fsc
            work5(i) = work5(i)/fsc
            work6(i) = work6(i)/fsc
 201     continue
      end if

C-----------------------------------------------------------------------
C   Compute the difference spectrum and its sigmas
C     DIFF(i) = R*OBJ(i;F,D) - REF(i;F)
C     SIGD(i) = SQRT[R^2*SIGO(i;F,D)^2 + SIGR(i;F)^2]
C   Loaded into the working arrays as:
C     WORK1(i) = WORK3(i) - WORK5(i)
C     WORK2(i) = SQRT(R*R*WORK4(i)*WORK4(i) + WORK6(i)*WORK6(i))
C

      do 202 i = 1, ntot
         work1(i) = work3(i) - work5(i)
         work2(i) = sqrt(r*r*work4(i)*work4(i) + work6(i)*work6(i))
 202  continue

C-----------------------------------------------------------------------
C   Open the output file - modified to dump the data in the form
C   favored at OSU, namely:
C
C     LAM(I)   DIFF(I)   DIFS(I)   OBJF(I)   OBJS(I)   REFF(I)   REFS(I)
C
C   where: "F" = "Fluxes" & "S" = "Sigmas"
C
 
      open (38, file=outfile, status='new',iostat=ierr)
C      open (38, file=outfile, recl=133, status='new',iostat=ierr)
      if (ierr .ne. 0) call io_errmsg(ierr,2,
     &     'Cannot open difference spectrum file',.true.)
         
c      if (f.ge.0.0) then
c         do 300 i = 1,ntot
c            write(38,*) wave(i),work3(i),work4(i),work3(i),work1(i),
c     &                  flux_1(i)-work3(i),flux_1(i)-work1(i)
c 300     continue
c      else
c         do 310 i = 1,ntot
c            write(38,*) wave(i),work3(i),work4(i),work5(i),flux_1(i),
c     &                  work5(i)-work3(i),flux_1(i)-work3(i)
c 310     continue
c      end if

      do 400 i = 1, ntot
         write(38,'(f9.2,6(1x,1pe12.5))',iostat=ierr) 
     &        wave(i), work1(i), work2(i), work3(i),
     &        work4(i), work5(i), work6(i)
         if (ierr .ne. 0) call io_errmsg(ierr,2,
     &        'Cannot to write differences file',.true.)
 400  continue

      close(38)
      return
      end
